from enum import Enum


class CharTestEnum(Enum):
    FIRST = 'first'
    SECOND = 'second'
    THIRD = 'third'


class CharLongValuesTestEnum(Enum):
    FIRST = 'first value'
    SECOND = 'second value'
    THIRD = 'third value'


class IntTestEnum(Enum):
    FIRST = 1
    SECOND = 2
    THIRD = 3
