# Generated by Django 2.2.2 on 2019-07-04 11:24

import django.contrib.postgres.fields
from django.db import migrations, models
import django_enum_choices.fields
import examples.enumerations


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enumerated_field', django_enum_choices.fields.EnumChoiceField(choices=[('a', 'a'), ('b', 'b')], enum_class=examples.enumerations.MyEnum, max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='MyModelMultiple',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enumerated_field', django.contrib.postgres.fields.ArrayField(base_field=django_enum_choices.fields.EnumChoiceField(choices=[('a', 'a'), ('b', 'b')], enum_class=examples.enumerations.MyEnum, max_length=1), size=None)),
            ],
        ),
    ]
