# Generated by Django 2.2.3 on 2019-07-19 12:38

from django.db import migrations, models
import django_enum_choices.fields
import examples.enumerations
import examples.models


class Migration(migrations.Migration):

    dependencies = [
        ('examples', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomReadableValueEnumModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enumerated_field', django_enum_choices.fields.EnumChoiceField(choice_builder=examples.models.choice_builder, choices=[('a', 'A'), ('b', 'B')], enum_class=examples.enumerations.MyEnum, max_length=1)),
            ],
        ),
    ]
