# Generated by Django 2.2.2 on 2019-08-24 17:31

from django.db import migrations, models
import django_enum_choices.choice_builders
import django_enum_choices.fields
import test_models.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SomeModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('choices', django_enum_choices.fields.EnumChoiceField(choice_builder=django_enum_choices.choice_builders.value_value, choices=[('a', 'a'), ('aa', 'aa'), ('aaa', 'aaa')], enum_class=test_models.models.ChoicesA, max_length=3)),
                ('other_choices', django_enum_choices.fields.EnumChoiceField(choice_builder=django_enum_choices.choice_builders.value_value, choices=[('a', 'a')], enum_class=test_models.models.ChoicesB, max_length=1)),
            ],
        ),
    ]
